@@ -22,10 +22,13 @@
 #include "galahad/glhd_public.h"
 #endif
 
+#ifdef GALLIUM_NOOP
+#include "noop/noop_public.h"
+#endif
+
 static INLINE struct pipe_screen *
 debug_screen_wrap(struct pipe_screen *screen)
 {
-
 #if defined(GALLIUM_RBUG)
    screen = rbug_screen_create(screen);
 #endif
@@ -38,6 +41,10 @@ debug_screen_wrap(struct pipe_screen *screen)
    screen = galahad_screen_create(screen);
 #endif
 
+#if defined(GALLIUM_NOOP)
+   screen = noop_screen_create(screen);
+#endif
+
    return screen;
 }
 
