@@ -54,13 +54,13 @@ extern void GLAPIENTRY
 _mesa_GetFixedv(GLenum pname, GLfixed *params);
 
 extern void GLAPIENTRY
-_mesa_GetBooleanIndexedv( GLenum pname, GLuint index, GLboolean *params );
+_mesa_GetBooleani_v( GLenum pname, GLuint index, GLboolean *params );
 
 extern void GLAPIENTRY
-_mesa_GetIntegerIndexedv( GLenum pname, GLuint index, GLint *params );
+_mesa_GetIntegeri_v( GLenum pname, GLuint index, GLint *params );
 
 extern void GLAPIENTRY
-_mesa_GetInteger64Indexedv( GLenum pname, GLuint index, GLint64 *params );
+_mesa_GetInteger64i_v( GLenum pname, GLuint index, GLint64 *params );
 
 extern void GLAPIENTRY
 _mesa_GetPointerv( GLenum pname, GLvoid **params );
@@ -74,4 +74,8 @@ _mesa_GetStringi(GLenum name, GLuint index);
 extern GLenum GLAPIENTRY
 _mesa_GetError( void );
 
+/* GL_ARB_robustness */
+extern GLenum GLAPIENTRY
+_mesa_GetGraphicsResetStatusARB( void );
+
 #endif
