@@ -26,8 +26,8 @@
  *    Chia-I Wu <olv@lunarg.com>
  */
 
+#include "main/mfeatures.h"
 #include "main/texobj.h"
-#include "main/texfetch.h"
 #include "main/teximage.h"
 #include "util/u_inlines.h"
 #include "util/u_format.h"
@@ -38,8 +38,6 @@
 #include "st_format.h"
 #include "st_manager.h"
 
-#if FEATURE_OES_EGL_image
-
 /**
  * Return the base format just like _mesa_base_fbo_format does.
  */
@@ -53,7 +51,7 @@ st_pipe_format_to_base_format(enum pipe_format format)
          base_format = GL_DEPTH_STENCIL;
       }
       else {
-         if (format == PIPE_FORMAT_S8_USCALED)
+         if (format == PIPE_FORMAT_S8_UINT)
             base_format = GL_STENCIL_INDEX;
          else
             base_format = GL_DEPTH_COMPONENT;
@@ -71,22 +69,19 @@ st_pipe_format_to_base_format(enum pipe_format format)
 }
 
 static void
-st_egl_image_target_renderbuffer_storage(GLcontext *ctx,
+st_egl_image_target_renderbuffer_storage(struct gl_context *ctx,
 					 struct gl_renderbuffer *rb,
 					 GLeglImageOES image_handle)
 {
    struct st_context *st = st_context(ctx);
    struct st_renderbuffer *strb = st_renderbuffer(rb);
    struct pipe_surface *ps;
-   unsigned usage;
 
-   usage = PIPE_BIND_RENDER_TARGET;
-   ps = st_manager_get_egl_image_surface(st, (void *) image_handle, usage);
+   ps = st_manager_get_egl_image_surface(st, (void *) image_handle);
    if (ps) {
       strb->Base.Width = ps->width;
       strb->Base.Height = ps->height;
       strb->Base.Format = st_pipe_format_to_mesa_format(ps->format);
-      strb->Base.DataType = st_format_datatype(ps->format);
       strb->Base._BaseFormat = st_pipe_format_to_base_format(ps->format);
       strb->Base.InternalFormat = strb->Base._BaseFormat;
 
@@ -98,7 +93,7 @@ st_egl_image_target_renderbuffer_storage(GLcontext *ctx,
 }
 
 static void
-st_bind_surface(GLcontext *ctx, GLenum target,
+st_bind_surface(struct gl_context *ctx, GLenum target,
                 struct gl_texture_object *texObj,
                 struct gl_texture_image *texImage,
                 struct pipe_surface *ps)
@@ -106,6 +101,7 @@ st_bind_surface(GLcontext *ctx, GLenum target,
    struct st_texture_object *stObj;
    struct st_texture_image *stImage;
    GLenum internalFormat;
+   gl_format texFormat;
 
    /* map pipe format to base format */
    if (util_format_get_component_bits(ps->format, UTIL_FORMAT_COLORSPACE_RGB, 3) > 0)
@@ -122,10 +118,11 @@ st_bind_surface(GLcontext *ctx, GLenum target,
       stObj->surface_based = GL_TRUE;
    }
 
-   _mesa_init_teximage_fields(ctx, target, texImage,
-                              ps->width, ps->height, 1, 0, internalFormat);
-   texImage->TexFormat = st_pipe_format_to_mesa_format(ps->format);
-   _mesa_set_fetch_functions(texImage, 2);
+   texFormat = st_pipe_format_to_mesa_format(ps->format);
+
+   _mesa_init_teximage_fields(ctx, texImage,
+                              ps->width, ps->height, 1, 0, internalFormat,
+                              texFormat);
 
    /* FIXME create a non-default sampler view from the pipe_surface? */
    pipe_resource_reference(&stObj->pt, ps->texture);
@@ -140,17 +137,15 @@ st_bind_surface(GLcontext *ctx, GLenum target,
 }
 
 static void
-st_egl_image_target_texture_2d(GLcontext *ctx, GLenum target,
+st_egl_image_target_texture_2d(struct gl_context *ctx, GLenum target,
 			       struct gl_texture_object *texObj,
 			       struct gl_texture_image *texImage,
 			       GLeglImageOES image_handle)
 {
    struct st_context *st = st_context(ctx);
    struct pipe_surface *ps;
-   unsigned usage;
 
-   usage = PIPE_BIND_SAMPLER_VIEW;
-   ps = st_manager_get_egl_image_surface(st, (void *) image_handle, usage);
+   ps = st_manager_get_egl_image_surface(st, (void *) image_handle);
    if (ps) {
       st_bind_surface(ctx, target, texObj, texImage, ps);
       pipe_surface_reference(&ps, NULL);
@@ -163,5 +158,3 @@ st_init_eglimage_functions(struct dd_function_table *functions)
    functions->EGLImageTargetTexture2D = st_egl_image_target_texture_2d;
    functions->EGLImageTargetRenderbufferStorage = st_egl_image_target_renderbuffer_storage;
 }
-
-#endif /* FEATURE_OES_EGL_image */
